
################################################
### GO ANALYSIS packages

### loading up the packages

if (!require("GOstats")) {
  source("https://bioconductor.org/biocLite.R")
  biocLite("GOstats")
  library(GOstats)
}

if (!require("GSEABase")) {
  source("https://bioconductor.org/biocLite.R")
  biocLite("GSEABase")
  library(GSEABase)
}

if (!require("treemap")) {
  install.packages("treemap", dependencies = TRUE)
  library(treemap)
}

################################################
### reading in the GO terms
### and sorting out for GOStats

GO_annotations <- read.table("Data/Nasonia_vitripennis_Annotation/Gene_GO_Nasonia_trinotate")

GO_annotations[,3] <- paste("IEA")

names(GO_annotations) <- c("genes","GOIds","evi")

GO_annotations[,3] <- paste("IEA")

GO_annotations <- GO_annotations[c(2,3,1)]


################################################
### creating a GO frame object and the 
### gene set collection object
### creating gene universe

GO_frame <- GOFrame(GO_annotations,organism = "Nasonia vitripennis")

goAllFrame <- GOAllFrame(GO_frame)

gsc <- GeneSetCollection(goAllFrame, setType = GOCollection())

universe <- as.vector(unique(GO_annotations[,3]))


################################################
### getting the list of genes of interest eg
### differentially expressed Loci

# Need to rerun for each different variable eg DE genes altsplice and DE and Alt splice only

A <- "Data/DE_Genes_host"
B <- "Data/DE_and_Splice"
C <- "Data/Spliced_no_DE"

# Just change from A, B or C depending on which one you want to analyse

DE_Genes <- read.table(C,header = T)

# change class to a vector
DE_Genes <- as.vector(DE_Genes[,1])

# Keeping only DE genes that been annotated

DE_Genes_A <- DE_Genes[DE_Genes %in% universe]


################################################
### Setting up the parameters for the 
### hypergeometric test
### function which produces list of different types of go analysis 
### eg Molecular function over and under respresentes GO terms

Get_GO_params_all <- function(genes_of_i,universe,pvalue_cut){
  
  onto_terms <- c("BP","CC","MF")
  directions <- c("over","under")
  param_list <- list()
  name_1 <- list()
  for(i in 1:3){
    for(j in 1:2){
      name_1 <- c(name_1,paste(onto_terms[i],directions[j],sep = "_"))
      parameters <- GSEAGOHyperGParams(name="Nasonia Hygeo params",
                                       geneSetCollection = gsc,
                                       universeGeneIds = universe,
                                       geneIds = DE_Genes_A,
                                       ontology = paste(onto_terms[i]),
                                       pvalueCutoff = pvalue_cut,
                                       conditional = T,testDirection = paste(directions[j]))
      
      
      
      param_list <- c(param_list,parameters)
      
      
    }
  }
  
  names(param_list) <- name_1
  return(param_list)
}

param_list <- Get_GO_params_all(genes_of_i = DE_Genes_A,universe = universe,
                                pvalue_cut = 0.05)


################################################
### conducting the hypergeometric test
### using the paramters set
### produces a list of results from the list of paramters


Hyper_G_test <- function(param_list){
  
  Hyper_G_list <- list()
  
  for(i in 1:length(param_list)){
    
    res <- hyperGTest(param_list[[i]])
    Hyper_G_list <- c(Hyper_G_list,res)
    
  }
  
  names(Hyper_G_list) <- names(param_list)
  return(Hyper_G_list)
}


GO_enrichment <- Hyper_G_test(param_list = param_list)

### P adjustment value to incorporate with p.adjust function

################################################
### in order to get the desired GO enrichment
### use summary function on element of
### list which is the enrichment you want to
### look at eg Molcular function overrepresentation
### also using a fdr adjusment.  Can use other adjusments
### but because they so stringent reliable to have no significant terms


Result <- summary(GO_enrichment[["MF_over"]])

Result_FDR <- Result[p.adjust(Result$Pvalue,method = "fdr") < 0.05,]

### take first two coloumns into revigo website

REVIGO <- Result_FDR[,1:2]


# Then write the table out for the appropriate analysis for MF and BP

# DE genes

write.table(Result,"Data/DE_Genes_GO_enrichment_BP",row.names = F , col.names = T)
write.table(REVIGO,"Data/DE_Genes_GO_enrichment_BP_REVIGO",row.names = F , col.names = F,quote = F)

write.table(Result,"Data/DE_Genes_GO_enrichment_MF",row.names = F , col.names = T)
write.table(REVIGO,"Data/DE_Genes_GO_enrichment_MF_REVIGO",row.names = F , col.names = F,quote = F)

# DE and Splice

write.table(Result,"Data/DE_and Splice_Genes_GO_enrichment_BP",row.names = F , col.names = T)
write.table(REVIGO,"Data/DE_and_Splice_Genes_GO_enrichment_BP_REVIGO",row.names = F , col.names = F,quote = F)

write.table(Result,"Data/DE_and Splice_Genes_GO_enrichment_MF",row.names = F , col.names = T)
write.table(REVIGO,"Data/DE_and_Splice_Genes_GO_enrichment_MF_REVIGO",row.names = F , col.names = F,quote = F)

# Splice only

write.table(Result,"Data/Splice_only_Genes_GO_enrichment_BP",row.names = F , col.names = T)
write.table(REVIGO,"Data/Splice_only_Genes_GO_enrichment_BP_REVIGO",row.names = F , col.names = F,quote = F)

write.table(Result,"Data/Splice_only_Genes_GO_enrichment_MF",row.names = F , col.names = T)
write.table(REVIGO,"Data/Splice_only_Genes_GO_enrichment_MF_REVIGO",row.names = F , col.names = F,quote = F)
